var newCache = require("./cache"),
    enabled = require('./cache/enabled'),
    debug = require('./debug')('cached');

module.exports = cached;

function cached (name, fn) {
  if (!enabled) return fn;

  return create(name, fn);
}

function cacheLater (cache, key, value) {
  debug('%s scheduled to save.', key);

  process.nextTick(function () {
    cache.set(key, JSON.stringify(value));
  });
}

function create (name, fn) {
  var cache = newCache(name);

  debug('Created cache for %s', name);

  return function () {
    var params = Array.prototype.slice.call(arguments, 0, -1);
    var callback = arguments[arguments.length - 1];
    return get(fn, cache, params, callback);
  };
}

function get (fn, cache, params, callback) {
  var key = JSON.stringify(params);

  cache.get(key, function (error, result) {
    if (result) {
      debug('Returning %s from cache.', key);
      return callback(undefined, JSON.parse(result));
    }

    debug('%s not found in cache. Reading from the actual resource.', key);

    params.push(function (error, result) {
      callback(error, result);
      if (!error && result) cacheLater(cache, key, result);
    });

    fn.apply(undefined, params);
  });
}
